#include "CentroidCluster.h"
#include "Domain_process.h"

/******************************************************************/
/* Function: float **GetCentroid						*/
/******************************************************************/
float **GetCentroid(float **wave_slope,int d_len,int gene_num,int *centroid_num)
{
	int i,j,count,cen_id;
	int num_centroid,*id,flag;
	float **centroid,**centroid_new;
	float dist,min_dist;
	int *indicate;
	
	if ((indicate=(int *)calloc(gene_num,sizeof(int)))==NULL)
	{printf("Error: calloc to id\n"); exit(0);}
	if ((id=(int *)calloc(gene_num,sizeof(int)))==NULL)
	{printf("Error: calloc to id\n"); exit(0);}

	for (i=0;i<gene_num;i++)
	{
		if (indicate[i]==0)
		{
			indicate[i]=1;
			id[i]=i;
			for (j=i+1;j<gene_num;j++)
			{		
				dist=SquareEuclideanDistance(wave_slope[i],wave_slope[j],d_len);
				if (dist<1.e-10)	{indicate[j]=2;	id[j]=i;}
			}
	
		}
	}
	count=0;
	for (i=0;i<gene_num;i++)
	{
		if (indicate[i]==0)	{printf("indicate=0\n"); exit(0);}
		else if (indicate[i]==1)	count++;
	}		

	num_centroid=count;
	if ((centroid=(float **)calloc(num_centroid,sizeof(float *)))==NULL)
	{printf("Error: calloc to centroid\n"); exit(0);}

	cen_id=0;	
	for (i=0;i<gene_num;i++)
	{
		if (indicate[i]==1)
		{
			if ((centroid[cen_id]=(float *)calloc(d_len,sizeof(float)))==NULL)
			{printf("Error: calloc to centroid\n"); exit(0);}
			for (j=0;j<d_len;j++)	centroid[cen_id][j]=wave_slope[i][j];
			cen_id++;	
		}
	
	}

	if (cen_id!=num_centroid)	{printf("cen_id(%d)!=num_centroid(%d)\n",cen_id,num_centroid);	exit(0);}
	
	free(indicate);
	/****Allocate id*****/
	for (i=0;i<gene_num;i++)
	{
		min_dist=1000.;
		for (j=0;j<num_centroid;j++)
		{
			dist=SquareEuclideanDistance(wave_slope[i],centroid[j],d_len);
			if (min_dist>dist)	{min_dist=dist;	id[i]=j;}
		}
	}
	
	/*************** Get Cluster centroids *******************/

	flag=CheckDistance(centroid,num_centroid,d_len);
	
	while (flag==1 && num_centroid!=1)
	{
		ChangeID(id,wave_slope,centroid,d_len,gene_num,num_centroid);
		centroid_new=Cluster(wave_slope,centroid,d_len,gene_num,num_centroid,id);


		for (i=0;i<num_centroid;i++)	free(centroid[i]);	free(centroid);
		
		num_centroid--;
		if ((centroid=(float **)calloc(num_centroid,sizeof(float *)))==NULL)
		{printf("Error: calloc to centroid\n"); exit(0);}
		for (i=0;i<num_centroid;i++)
		{
			if ((centroid[i]=(float *)calloc(d_len,sizeof(float)))==NULL)
			{printf("Error: calloc to centroid\n"); exit(0);}
			for (j=0;j<d_len;j++)
				centroid[i][j]=centroid_new[i][j];
			free(centroid_new[i]);
		}
		free(centroid_new);
		
		flag=CheckDistance(centroid,num_centroid,d_len);
	}

	free(id);
	
	*centroid_num=num_centroid;
	
	return centroid;
}

/******************************************************************/
/* Function: ChangeID						*/
/******************************************************************/
void ChangeID(int *id,float **wave_slope,float **centroid,int d_len,int gene_num,int num_centroid)
{
	int i,j,*count,select;
	float min_dist,dist;

	if ((count=(int *)calloc(num_centroid,sizeof(int)))==NULL)
	{printf("Error: count\n"); exit(0);}
	
	for (i=0;i<gene_num;i++)
	{
		count[id[i]]++;
	}


	for (i=0;i<num_centroid;i++)
	{
		if (count[i]==0)
			printf("b4,count[%d]=0\n",i);
		count[i]=0;
	}

	for (i=0;i<gene_num;i++)
	{
		min_dist=1000.;
		for (j=0;j<num_centroid;j++)
		{

			dist=SquareEuclideanDistance(centroid[j],wave_slope[i],d_len);
			if (min_dist>dist)
				{min_dist=dist;	select=j;}

		}
		id[i]=select;
		count[id[i]]++;

	}

	for (i=0;i<num_centroid;i++)
		if (count[i]==0)
			printf("after,count[%d]=0\n",i);
	free(count);	
	
}


/******************************************************************/
/* Function: float **Cluster						*/
/******************************************************************/
float **Cluster(float **x, float **x_centroid, int len, int num_x, int num_centroid, int *idx)
{
	int i,j,cen1,cen2,*count;
	float dist,min_dist=1000.;
	float **centroid_new;

	for (i=0;i<num_centroid;i++)
	{
		for (j=i+1;j<num_centroid;j++)
		{
			dist=SquareEuclideanDistance(x_centroid[i],x_centroid[j],len);
			if (min_dist>dist)
			{
				min_dist=dist;
				cen1=i;
				cen2=j;
			}
		}
	}
		

	if ((centroid_new=(float **)calloc(num_centroid-1,sizeof(float *)))==NULL)
	{printf("Error: calloc to centroid_new\n"); exit(0);}

	for (i=0;i<num_centroid-1;i++)
		if ((centroid_new[i]=(float *)calloc(len,sizeof(float)))==NULL)
		{printf("Error: calloc to centroid_new\n"); exit(0);}
	

	/*********** Relocate members to cluster **********************/
	for (i=0;i<num_x;i++)
	{
		if (idx[i]==cen2)	idx[i]=cen1;
		else if (idx[i]>cen2)	idx[i]--;
	}




	if ((count=(int *)calloc(num_centroid-1,sizeof(int)))==NULL)
	{printf("Error: calloc to count\n"); exit(0);}

	for (i=0;i<num_x;i++)
	{
		count[idx[i]]++;
		for (j=0;j<len;j++)
		{
			centroid_new[idx[i]][j]+=x[i][j];			
		}
	}


//CheckPoint
	for (i=0;i<num_centroid-1;i++)	if (count[i]==0)	
	{printf("Error2: count[%d]=%d\ncen1=%d\tcen2=%d\n",i,count[i],cen1,cen2); exit(0);}

	for (i=0;i<num_centroid-1;i++)
	{
		for (j=0;j<len;j++)
			centroid_new[i][j]/=count[i];
	}

	free(count);
	return centroid_new;

}

/******************************************************************/
/* Function: int CheckDistance						*/
/******************************************************************/

int CheckDistance(float **x,int num,int len)
{
	int i,j,k;
	float dist;

	for (i=0;i<num;i++)
	{
		for (j=i+1;j<num;j++)
		{
			dist=SquareEuclideanDistance(x[i],x[j],len);
			
			if (dist<CUTOFF)
				return 1;
		}
	}

	return 0;	
}

float *GetSingleCentroid(float **wave_slope,int len,int gene_num)
{
	float *centroid;
	int i,j;
	
	if ((centroid=(float *)calloc(len,sizeof(float)))==NULL)
	{printf("Error: calloc to centroid\n"); exit(0);}

	for (i=0;i<len;i++)
	{
		centroid[i]=0.;
		for (j=0;j<gene_num;j++)
		{
			centroid[i]+=wave_slope[j][i];	
		}
		centroid[i]/=gene_num;
	}	

	return centroid;
}

