#include "HMM_build.h"

void main(int argc, char **argv)
{
	int i,j,k,z;
	FILE *fid;

	if (argc!=3)
	{printf("Usage: HMM_build <train set> <profile outfile>\n");	exit(0);}

	//ftmp=fopen("tmp","wt");
	Initialize();

	TrainHMM(argv[1]);
//	printf("Training Done\n");		

	/***********Print profiles into profile file ********************/
	/* Format: 							*/
	/* DOMAIN_LEN							*/
	/* d_centroid_num	d_len					*/
	/* d_centroids							*/
	/****************************************************************/

	if ((fid=fopen(argv[2],"wt"))==NULL)
	{printf("Error opening %s\n",argv[2]);	exit(0);}

	fprintf(fid,"%d\n",DOMAIN_NUM);
		
	for (i=0;i<DOMAIN_NUM;i++)
	{	
		fprintf(fid,"%d\n",d_len[i]);
		for (j=0;j<num_indices;j++)
		{
			for (k=0;k<d_len[i];k++)
				fprintf(fid,"%f\t",d_centroid[i][j][k]);
			fprintf(fid,"\n");
		}
		for (j=0;j<num_indices;j++)
		{	
			fprintf(fid,"%f\t%f\n",d_gam[i][j].a,d_gam[i][j].b);
		}
	}
	fclose(fid);
	
        for (i=0;i<DOMAIN_NUM;i++)      free(d_gam[i]); free(d_gam);
        for (i=0;i<DOMAIN_NUM;i++)      {for (j=0;j<num_indices;j++)    free(d_centroid[i][j]);	free(d_centroid[i]);}
	free(d_centroid);
	free(d_len);
//	fclose(ftmp);
}


/****************************************************************************************/
/* Function: Initialize									*/
/* Purpose: Initialize 'd_len'								*/
/****************************************************************************************/
void Initialize(void)
{
	int i,j,k;

	if ((d_len=(int *)calloc(DOMAIN_NUM,sizeof(int)))==NULL)
	{printf("Error: calloc to d_len\n"); exit(0);}
	d_len[0]=22;	d_len[1]=19;	d_len[2]=15;	d_len[3]=15;
	d_len[4]=24;	d_len[5]=23;	d_len[6]=18;	d_len[7]=15;
	d_len[8]=17;	d_len[9]=22;	d_len[10]=20;	d_len[11]=26;
	
	
}


/****************************************************************************************/
/* Function: TrainHMM									*/
/* Purpose: Read in sequences for 12 domains, 'get d_centroid', 'd_centroid_num' 	*/
/****************************************************************************************/

void TrainHMM(char *trainset)
{
	int i,j,k,gene_num,index_num,centroid_num,select;
	FASTATYPE *seq;
	char *filename;
	float **aa_index,**wave,**sig,**win,**centroid,**wave_slope;
	float dist,min_dist;
	int index_i;

	/************************************************/
	/* Read in domain sequence, get 'd_centroid'	*/
	/************************************************/
	
	aa_index=GetIndexKey("index36.out",&index_num);

	if ((d_centroid=(float ***)calloc(DOMAIN_NUM,sizeof(float **)))==NULL)
	{printf("Error: calloc to d_centroid\n"); exit(0);}
	if ((d_gam=(GAMMATYPE **)calloc(DOMAIN_NUM,sizeof(GAMMATYPE *)))==NULL)
	{printf("Error: calloc to d_gam\n");	exit(0);}

	for (i=0;i<DOMAIN_NUM;i++)
	{
//		filename=GetFilename(i,trainset);
//		seq=fasta(filename,&gene_num);
		if ((d_centroid[i]=(float **)calloc(num_indices,sizeof(float *)))==NULL)
		{printf("Error: calloc to d_centroid\n"); exit(0);}
		if ((d_gam[i]=(GAMMATYPE *)calloc(num_indices,sizeof(GAMMATYPE)))==NULL)
		{printf("Error: calloc to d_gam\n"); exit(0);}
		
		for (index_i=0;index_i<num_indices;index_i++)
		{
                filename=GetFilename(i,trainset);
                seq=fasta(filename,&gene_num);

//		printf("Domain %d\tindex=%d\n",i+1,index_i+1);
		/****Get signal ****/
		if ((sig=(float **)calloc(gene_num,sizeof(float *)))==NULL)
		{printf("Error: calloc sig\n"); exit(0);}
                if ((win=(float **)calloc(gene_num,sizeof(float *)))==NULL)
                {printf("Error: calloc sig\n"); exit(0);}
		if ((wave=(float **)calloc(gene_num,sizeof(float *)))==NULL)
		{printf("Error: calloc wave\n"); exit(0);}
		if ((wave_slope=(float **)calloc(gene_num,sizeof(float *)))==NULL)
		{printf("Error: calloc wave_slope\n"); exit(0);}

		for (j=0;j<gene_num;j++)
		{
			sig[j]=GetSingleIndexSignal(seq[j].seq,seq[j].seq_len,aa_index[num_indices*i+index_i]);
			win[j]=SigWinSingle(sig[j],seq[j].seq_len);
			wave[j]=GetWaveSingle(sig[j],seq[j].seq_len);
			wave_slope[j]=GetSlopeSingle(wave[j],seq[j].seq_len);
		}

		/****Get centroid****/
		d_centroid[i][index_i]=GetSingleCentroid(wave_slope,d_len[i],gene_num);

		/****Get gamma parameter estimate ****/
		d_gam[i][index_i]=FitDomainToGamma(d_centroid[i][index_i],wave_slope,gene_num,d_len[i]);
			
		for (j=0;j<gene_num;j++)	free(sig[j]);	free(sig);
                for (j=0;j<gene_num;j++)        free(win[j]);   free(win);
		for (j=0;j<gene_num;j++)	free(wave[j]);	free(wave);
		for (j=0;j<gene_num;j++)	free(wave_slope[j]);	free(wave_slope);
		free(filename);
		for (j=0;j<gene_num;j++)      {free(seq[j].name);     free(seq[j].seq);}      free(seq);
		}//index_i
		
//		free(filename);	
//		printf("break1\n");
		//for (j=0;j<gene_num;j++)	{free(seq[j].name);	free(seq[j].seq);}	free(seq);		
	
	}



	for (i=0;i<index_num;i++)	free(aa_index[i]);	free(aa_index);

}


/****************************************************************/
/* Function: GetInitProb					*/
/* Purpose: Get Init  probability				*/
/****************************************************************/
float *GetInitProb(int *curr_select,int gene_num,int centroid_num)
{
	int i,j;
	float *count;


	if ((count=(float *)calloc(centroid_num,sizeof(float)))==NULL)
	{printf("Error: calloc to count\n"); exit(0);}

	for (i=0;i<gene_num;i++)
	{
		if (curr_select[i]>=centroid_num)
		{printf("Error: curr_select=%d\tcentroid_num=%d\n",curr_select[i],centroid_num); exit(0);}
		count[curr_select[i]]+=1.;
	}
	for (i=0;i<centroid_num;i++)	count[i]/=gene_num;

	return count;
}

/****************************************************************/
/* Function: GetTranProb					*/
/* Purpose: Get Transition probability				*/
/****************************************************************/
float **GetTranProb(int *prev,int *curr,int gene_num,int prev_centroid_num,int curr_centroid_num)
{
	int i,j;
	float **prob,*count;

	if ((count=(float *)calloc(prev_centroid_num,sizeof(float)))==NULL)
	{printf("Error: calloc to prob\n"); exit(0);}
	if ((prob=(float **)calloc(prev_centroid_num,sizeof(float *)))==NULL)
	{printf("Error: calloc to prob\n"); exit(0);}
	for (i=0;i<prev_centroid_num;i++)
		if ((prob[i]=(float *)calloc(curr_centroid_num,sizeof(float)))==NULL)
		{printf("Error: calloc to prob\n"); exit(0);}

	for (i=0;i<gene_num;i++)
		prob[prev[i]][curr[i]]+=1.;

	for (i=0;i<prev_centroid_num;i++)
	{
		for (j=0;j<curr_centroid_num;j++)
			count[i]+=prob[i][j];
	}
	for (i=0;i<prev_centroid_num;i++)	
	{
		if (count[i]<1.e-10)	{printf("Error in count[%d]=0\n",i); exit(0);}
		for (j=0;j<curr_centroid_num;j++)	prob[i][j]/=count[i];
	}
	
	return prob;
}


/****************************************************************/
/* Function: GetEmitProb					*/
/* Purpose: Get Emission probability				*/
/****************************************************************/
float *GetEmitProb(int *select,int gene_num,int centroid_num)
{
	int i,j,k;
	float *prob;

	if ((prob=(float *)calloc(centroid_num,sizeof(float)))==NULL)
	{printf("Error: calloc to prob\n"); exit(0);}

	for (i=0;i<gene_num;i++)
		prob[select[i]]+=1.;
	for (i=0;i<centroid_num;i++)	prob[i]/=gene_num;

	return prob;
}

/****************************************************************/
/* Function: GAMMATYPE *FitDomainToGamma			*/
/* Purpose: Get gamma parameter estimates for each profile	*/
/****************************************************************/

GAMMATYPE FitDomainToGamma(float *centroid_slope,float **wave_slope,int gene_num,int len)
{
	int i,j,*id,*count,*id_wave;
	float ***wave,a,b,dist_min,dist;
	GAMMATYPE gam;


        EstimateGamParameter(&a,&b,wave_slope,centroid_slope,len,gene_num,gene_num);
	gam.a=a;	gam.b=b;


	return gam;
}


/****************************************************************/
/* Function: char *GetFilename(int domain)			*/
/* Purpose: Get filenames of domains seq			*/
/****************************************************************/
char *GetFilename(int domain,char *input_file)
{
	char *filename;

	if ((filename=(char *)calloc(20,sizeof(char)))==NULL)
	{printf("Error: calloc to filename\n"); exit(0);}

	strcpy(filename,input_file);
	switch(domain+1)
	{
		case 1:
			strcat(filename,".d1");
			break;
		case 2:
			strcat(filename,".d2");
			break;
		case 3:
			strcat(filename,".d3");
			break;
		case 4: 
			strcat(filename,".d4");
			break;
		case 5: 
			strcat(filename,".d5");
			break;
		case 6: 
			strcat(filename,".d6a");
			break;
		case 7:
			strcat(filename,".d6b");
			break;
		case 8: 
			strcat(filename,".d7");
			break;
		case 9: 
			strcat(filename,".d8");
			break;
		case 10: 
			strcat(filename,".d9");
			break;
		case 11: 
			strcat(filename,".d10");
			break;
		case 12: 
			strcat(filename,".d11");
			break;
		default:
			printf("Error: choice not in range\n");
			exit(0);
	}

	return filename;
}
