#include "gamma.h"
#include "Domain_process.h"

/*********************************************************/
/* Purpose: EstimateGamParameter			*/
/*********************************************************/
void EstimateGamParameter(float *a, float *b, float **wave_slope, float *profile_slope, int domain_len, int gene_num,int count)
{
	int i,j,id=0;
	float *ddp,alpha,beta;


	if ((ddp=(float *)calloc(count,sizeof(float)))==NULL)
	{printf("Error: calloc to ddp\n"); exit(0);}

	for (i=0;i<gene_num;i++)
	{
		ddp[i]=SquareEuclideanDistance(wave_slope[i],profile_slope,domain_len);
		if (ddp[i]<1.e-10)	ddp[i]=1.e-5;
	}
	
	gamfit(ddp,count,&alpha,&beta);
	
	*a=alpha;	*b=beta;

	free(ddp);
}

/*********************************************************/
/* Function: gampdf			*/
/* Purpose: get pdf with given data, alpha and beta */
/*********************************************************/

float gampdf(float x,float a,float b)
{
	float y;

	y = (a - 1)*log(x)-(x/b)-gammaln(a)-a*log(b);
    	y = exp(y);
	
	return y;
}

float gammaln(float xx)
{

	float x,y,tmp,ser;
	static float cof[6]={76.18009172947146,-86.50532032941677,
	24.01409824083091,-1.231739572450155,
	0.1208650973866179e-2,-0.5395239384953e-5};
	int j;
	y=x=xx;
	tmp=x+5.5;
	tmp -= (x+0.5)*log(tmp);
	ser=1.000000000190015;
	for (j=0;j<=5;j++) ser += cof[j]/++y;
	return -tmp+log(2.5066282746310005*ser/x);
}

/*********************************************************/
/* Function: gamfit			*/
/* Purpose: Estimate alpha and beta using MLE */
/*********************************************************/
void gamfit(float *x,int N, float *alpha, float *beta)
{
	int i,flag=0;
	float xbar=0.,s2=0.,sumx=0.;
	float a,ahat, bhat, delta=0.00025, nll,nll_min;

	/**** Find mean and sd^2 ****/
	for (i=0;i<N;i++)		xbar+=x[i];
	xbar/=N;
	
	for (i=0;i<N;i++)	s2+=(x[i]-xbar)*(x[i]-xbar);
	s2/=(N-1);


	/**** Use Method of Moments estimates as starting for MLEs ****/
	ahat= xbar*xbar/s2;
	bhat = xbar/ahat;


	/** Ensure that ML is possible -- if not, fall back to MM **/
	for (i=0;i<N;i++)	
		if (fabs(x[i])<1.e-10)	{flag=1;	break;}

	if (flag==1)
	{
		printf("zeros in the data\n");
		exit(0);
	}
	else
	{
		if (ahat<3.)	
			a=ahat;		
		else 
			a=1.5;

		nll_min=negloglike(a,x,N);

		while (a<3.)
		{
			nll=negloglike(a,x,N);
			if (nll<nll_min)
			{
				ahat=a;
				nll_min=nll;
			}
			a+=delta;
		}
		while (a>0.)
		{
			nll=negloglike(a,x,N);
			if (nll<nll_min)
			{
				ahat=a;
				nll_min=nll;
			}
			a-=delta;
		}

	}

	for (i=0;i<N;i++)	sumx+=x[i];

	*alpha=ahat;
	bhat=sumx/(N*ahat);
	*beta=bhat;


}

/*********************************************************/
/* Function: negloglike			*/
/* Purpose: get negetive log likelihood for MLE */
/*********************************************************/
float negloglike(float alpha, float *x,int N)
{
	int i;
	float sumx=0.,sumlogx=0.;
	float nll;

	for (i=0;i<N;i++)	{sumx+=x[i];	sumlogx+=log(x[i]);}
	
	
nll=(1-alpha)*sumlogx+N*alpha*(1+log(sumx/(N*alpha)))+N*gammaln(alpha);

	return nll;			
}


